@@ -1,13 +1,10 @@
 package Benchmark::Perl::Formance;
-# git description: v0.32-1-g7d898fe
+# git description: v0.33-12-g7de25a8
 
 BEGIN {
   $Benchmark::Perl::Formance::AUTHORITY = 'cpan:SCHWIGON';
 }
-{
-  $Benchmark::Perl::Formance::VERSION = '0.33';
-}
-
+$Benchmark::Perl::Formance::VERSION = '0.34';
 use 5.008;
 
 use warnings;
@@ -25,6 +22,7 @@ use Data::DPath 'dpath', 'dpathi';
 use File::Find;
 use Storable "fd_retrieve", "store_fd";
 use Sys::Hostname;
+use FindBin qw($Bin);
 
 # comma separated list of default plugins
 my $DEFAULT_PLUGINS = join ",", qw(DPath
@@ -69,11 +67,9 @@ my $ALL_PLUGINS = join ",", qw(DPath
                                ThreadsShared
                              );
 
-our $DEFAULT_INDENT          = 0;
+our $scaling_script = "$Bin/benchmark-perlformance-set-stable-system";
 
-our $PROC_RANDOMIZE_VA_SPACE = "/proc/sys/kernel/randomize_va_space";
-our $PROC_DROP_CACHES        = "/proc/sys/vm/drop_caches";
-our $SYS_CPB                 = "/sys/devices/system/cpu/cpu0/cpufreq/cpb";
+our $DEFAULT_INDENT          = 0;
 
 my @run_plugins;
 
@@ -108,7 +104,8 @@ my %CONFIG_KEYS = (
                   );
 
 sub new {
-        bless {}, shift;
+        my ($class, %args) = @_;
+        bless { %args }, $class;
 }
 
 sub load_all_plugins
@@ -184,69 +181,39 @@ For more details see
 ';
 }
 
-sub set_proc
-{
-        my ($self, $file, $value) = @_;
-
-        if (! -e $file) {
-                print STDERR "# Could not find $file\n" if $self->{options}{verbose} >= 4;
-                return undef;
-        }
-        if (not defined $value) {
-                print STDERR "# No value given\n" if $self->{options}{verbose} >= 4;
-                return undef;
-        }
-
-        my $orig_value;
-        if (open (my $PROCFILE, "<", $file)) {
-                local $/ = undef;
-                $orig_value = <$PROCFILE>;
-                close $PROCFILE;
-        } else {
-                print STDERR "# Could not read old value from $file\n" if $self->{options}{verbose} >= 4;
-        }
-        chomp $orig_value if defined $orig_value;
-
-        if (open (my $PROCFILE, ">", $file)) {
-                print $PROCFILE $value;
-                close $PROCFILE;
-        } else {
-                print STDERR "# Could not write $value into $file\n" if $self->{options}{verbose} >= 4;
-        }
-
-        return $orig_value;
-}
-
 sub do_disk_sync {
-        my ($self) = @_;
-        system("sync");
+    system("sync ; sync");
 }
 
-# Try to stabilize a system.
-# - Classical disk sync
-# - Drop caches (http://linux-mm.org/Drop_Caches)
-# - Disable address space randomization (ASLR) (https://wiki.ubuntu.com/Security/Features)
-# - Disable "Core Performance Boost" (http://lkml.org/lkml/2010/3/22/333)
 sub prepare_stable_system
 {
         my ($self) = @_;
 
         my $orig_values;
-        if ($^O eq "linux") {
-                $orig_values->{aslr} = $self->set_proc ($PROC_RANDOMIZE_VA_SPACE, 0);
-                $orig_values->{cpb}  = $self->set_proc ($SYS_CPB, 0);
-                $self->do_disk_sync;
-                $self->set_proc ($PROC_DROP_CACHES, 1);
+        if ($self->{options}{stabilize_cpu} and $^O eq "linux") {
+                $self->{orig_system_values} = qx(sudo $scaling_script lo);
+                do_disk_sync();
         }
-        return $orig_values;
 }
 
 sub restore_stable_system
 {
         my ($self, $orig_values) = @_;
-        if ($^O eq "linux") {
-                $self->set_proc($PROC_RANDOMIZE_VA_SPACE, $orig_values->{aslr}) if defined $orig_values->{aslr};
-                $self->set_proc($SYS_CPB,                 $orig_values->{cpb} ) if defined $orig_values->{cpb};
+        if ($self->{options}{stabilize_cpu} and $^O eq "linux") {
+                if (open my $RESTORE, "|-", "sudo $scaling_script restore") {
+                    print $RESTORE $self->{orig_system_values};
+                    close $RESTORE;
+                }
+        }
+}
+
+sub prepare_fast_system
+{
+        my ($self) = @_;
+
+        my $orig_values;
+        if ($self->{options}{stabilize_cpu} and $^O eq "linux") {
+                $self->{orig_system_values} = qx(sudo $scaling_script hi);
         }
 }
 
@@ -277,10 +244,8 @@ sub run_plugin
                                 exit 0;
                         }
                         $0 = "benchmark-perl-formance-$pluginname";
-                        my $orig_values = $self->prepare_stable_system;
                         $res = &{"Benchmark::Perl::Formance::Plugin::${pluginname}::main"}($self->{options});
                         $res->{PLUGIN_VERSION} = ${"Benchmark::Perl::Formance::Plugin::${pluginname}::VERSION"};
-                        $self->restore_stable_system($orig_values);
                         store_fd($res, \*CHILD_WTR);
                         close CHILD_WTR;
                         exit 0;
@@ -304,7 +269,7 @@ sub run_plugin
 sub _perl_gitversion {
         my $perlpath = "$^X";
         $perlpath    =~ s,/[^/]*$,,;
-        my $perl_gitversion  = "$perlpath/perl-gitchangeset";
+        my $perl_gitversion  = "$perlpath/perl -MConfig -e 'print \$Config{bootstrap_perl_git_changeset}";
 
         if (-x $perl_gitversion) {
                 my $gitversion = qx!$perl_gitversion! ;
@@ -316,7 +281,7 @@ sub _perl_gitversion {
 sub _perl_gitdescribe {
         my $perlpath = "$^X";
         $perlpath    =~ s,/[^/]*$,,;
-        my $perl_gitdescribe  = "$perlpath/perl-gitdescribe";
+        my $perl_gitdescribe  = "$perlpath/perl -MConfig -e 'print \$Config{bootstrap_perl_git_describe}";
 
         if (-x $perl_gitdescribe) {
                 my $gitdescribe = qx!$perl_gitdescribe! ;
@@ -325,19 +290,19 @@ sub _perl_gitdescribe {
         }
 }
 
-sub _perl_codespeed_executable {
+sub _perl_symbolic_name {
         my $perlpath = "$^X";
         $perlpath    =~ s,/[^/]*$,,;
-        my $perl_codespeed_executable  = "$perlpath/perl-codespeed-executable";
+        my $perl_symbolic_name  = "$perlpath/perl -MConfig -e 'print \$Config{bootstrap_perl_symbolic_name}";
 
-        if (-x $perl_codespeed_executable) {
-                my $executable = qx!$perl_codespeed_executable! ;
+        if (-x $perl_symbolic_name) {
+                my $executable = qx!$perl_symbolic_name! ;
                 chomp $executable;
                 return $executable;
         }
 }
 
-sub _codespeed_optional_tag {
+sub _optional_tag {
         # only create tags for stable releases
         my $gitdescribe = _perl_gitdescribe;
         if ($gitdescribe =~ /^(v|perl-)?5\.(\d+)\.\d+$/) {
@@ -365,12 +330,12 @@ sub generate_codespeed_data
         my $len = max map { length } @run_plugins;
 
         my $codespeed_exe_suffix  = $self->{options}{cs_executable_suffix}  || $ENV{CODESPEED_EXE_SUFFIX}  || "";
-        my $codespeed_exe         = $self->{options}{cs_executable}         || _perl_codespeed_executable  || sprintf("perl-%s.%s%s",
+        my $codespeed_exe         = $self->{options}{cs_executable}         || _perl_symbolic_name  || sprintf("perl-%s.%s%s",
                                                                                                                       $Config{PERL_REVISION},
                                                                                                                       $Config{PERL_VERSION},
                                                                                                                       $codespeed_exe_suffix,
                                                                                                                      );
-        my $codespeed_project     = $self->{options}{cs_project}            || $ENV{CODESPEED_PROJECT}     || "perl";
+        my $codespeed_project     = $self->{options}{cs_project}            || $ENV{CODESPEED_PROJECT}     || "perl5";
         my $codespeed_branch      = $self->{options}{cs_branch}             || $ENV{CODESPEED_BRANCH}      || "default";
         my $codespeed_commitid    = $self->{options}{cs_commitid}           || $ENV{CODESPEED_COMMITID}    || $Config{git_commit_id} || _perl_gitversion || "no-commit";
         my $codespeed_environment = $self->{options}{cs_environment}        || $ENV{CODESPEED_ENVIRONMENT} || _get_hostname || "no-env";
@@ -381,7 +346,7 @@ sub generate_codespeed_data
                               commitid    => $codespeed_commitid,
                               environment => $codespeed_environment,
                               # do not add tag here, it seems not to be the correct API,
-                              # _codespeed_optional_tag
+                              # _optional_tag()
                              );
 
         foreach (sort @run_plugins) {
@@ -399,6 +364,52 @@ sub generate_codespeed_data
         return \@codespeed_entries;
 }
 
+sub generate_BenchmarkAnythingData_data
+{
+        my ($self, $RESULTS) = @_;
+
+        my @benchmarkanythingdata_entries = ();
+
+        my @run_plugins = $self->find_interesting_result_paths($RESULTS);
+        my $len = max map { length } @run_plugins;
+
+        my $benchmarkanythingdata_exe_suffix  = $self->{options}{cs_executable_suffix}  || $ENV{BENCHMARKANYTHINGDATA_EXE_SUFFIX}  || "";
+        my $benchmarkanythingdata_exe         = $self->{options}{cs_executable}         || _perl_symbolic_name  || sprintf("perl-%s.%s%s",
+                                                                                                                      $Config{PERL_REVISION},
+                                                                                                                      $Config{PERL_VERSION},
+                                                                                                                      $benchmarkanythingdata_exe_suffix,
+                                                                                                                     );
+        my $benchmarkanythingdata_project     = $self->{options}{cs_project}            || $ENV{BENCHMARKANYTHINGDATA_PROJECT}     || "perl";
+        my $benchmarkanythingdata_branch      = $self->{options}{cs_branch}             || $ENV{BENCHMARKANYTHINGDATA_BRANCH}      || "default";
+        my $benchmarkanythingdata_commitid    = $self->{options}{cs_commitid}           || $ENV{BENCHMARKANYTHINGDATA_COMMITID}    || $Config{git_commit_id} || _perl_gitversion || "no-commit";
+        my $benchmarkanythingdata_environment = $self->{options}{cs_environment}        || $ENV{BENCHMARKANYTHINGDATA_ENVIRONMENT} || _get_hostname || "no-env";
+
+        my %benchmarkanythingdata_meta = (
+                                          executable  => $benchmarkanythingdata_exe,
+                                          project     => $benchmarkanythingdata_project,
+                                          branch      => $benchmarkanythingdata_branch,
+                                          commitid    => $benchmarkanythingdata_commitid,
+                                          environment => $benchmarkanythingdata_environment,
+                                          _optional_tag()
+                                         );
+
+        my %bootstrap_perl_meta = map { ( "perlformance_$_" => $Config{$_} ) } grep { /^bootstrap_perl/ } keys %Config;
+
+        foreach (sort @run_plugins) {
+                no strict 'refs'; ## no critic
+                my @resultkeys = split(/\./);
+                my ($res) = dpath("/results/".join("/", map { qq("$_") } @resultkeys)."/Benchmark/*[0]")->match($RESULTS);
+                my $benchmark =  $self->{options}{fastmode} ? "$_(F)" : $_ ;
+                push @benchmarkanythingdata_entries, {
+                                                      %benchmarkanythingdata_meta,
+                                                      %bootstrap_perl_meta,
+                                                      NAME  => $benchmark,
+                                                      VALUE => ($res || undef),
+                                                     };
+        }
+        return \@benchmarkanythingdata_entries;
+}
+
 sub run {
         my ($self) = @_;
 
@@ -407,6 +418,7 @@ sub run {
         my $outstyle       = "summary";
         my $platforminfo   = 0;
         my $codespeed      = 0;
+        my $tapper         = 0;
         my $cs_executable_suffix = "";
         my $cs_executable        = "";
         my $cs_project           = "";
@@ -418,6 +430,7 @@ sub run {
         my $fastmode       = 0;
         my $useforks       = 0;
         my $quiet          = 0;
+        my $stabilize_cpu  = 0;
         my $plugins        = $DEFAULT_PLUGINS;
         my $indent         = $DEFAULT_INDENT;
         my $tapdescription = "";
@@ -434,9 +447,11 @@ sub run {
                              "fastmode"         => \$fastmode,
                              "version"          => \$version,
                              "useforks"         => \$useforks,
+                             "stabilize-cpu"    => \$stabilize_cpu,
                              "showconfig|c+"    => \$showconfig,
                              "platforminfo|p"   => \$platforminfo,
                              "codespeed"        => \$codespeed,
+                             "tapper"           => \$tapper,
                              "cs-executable-suffix=s" => \$cs_executable_suffix,
                              "cs-executable=s"  => \$cs_executable,
                              "cs-project=s"     => \$cs_project,
@@ -454,9 +469,11 @@ sub run {
                             outstyle       => $outstyle,
                             fastmode       => $fastmode,
                             useforks       => $useforks,
+                            stabilize_cpu  => $stabilize_cpu,
                             showconfig     => $showconfig,
                             platforminfo   => $platforminfo,
                             codespeed      => $codespeed,
+                            tapper         => $tapper,
                             cs_executable_suffix => $cs_executable_suffix,
                             cs_executable        => $cs_executable,
                             cs_project           => $cs_project,
@@ -487,12 +504,16 @@ sub run {
         my $before = gettimeofday();
         my %RESULTS;
         my @plugins = grep /\w/, split '\s*,\s*', $plugins;
+
+        $self->prepare_stable_system;
         foreach (@plugins)
         {
                 my @resultkeys = split(qr/::|\./, $_);
                 my $res = $self->run_plugin($_);
                 eval "\$RESULTS{results}{".join("}{", @resultkeys)."} = \$res"; ## no critic
         }
+        $self->prepare_fast_system; # simply set to max, as restore_stable_system() is no reliable approach anyway
+
         my $after  = gettimeofday();
         $RESULTS{perlformance}{overall_runtime}   = $after - $before;
         $RESULTS{perlformance}{config}{fastmode}  = $fastmode;
@@ -523,6 +544,12 @@ sub run {
                 $RESULTS{codespeed} = $self->generate_codespeed_data(\%RESULTS, $platform_info);
         }
 
+        # Tapper BenchmarkAnythingData blocks
+        if ($tapper)
+        {
+                $RESULTS{BenchmarkAnythingData} = $self->generate_BenchmarkAnythingData_data(\%RESULTS, $platform_info);
+        }
+
         unbless (\%RESULTS);
         return \%RESULTS;
 }
@@ -594,7 +621,7 @@ sub print_results
 
 1;
 
-
+__END__
 
 =pod
 
@@ -702,13 +729,9 @@ Steffen Schwigon <ss5@renormalist.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Steffen Schwigon.
+This software is copyright (c) 2015 by Steffen Schwigon.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-
-__END__
-
